@@ -1,4 +1,4 @@
-/*  $Id: main-window.cpp,v 1.29 2010/07/04 05:49:51 sarrazip Exp $
+/*  $Id: main-window.cpp,v 1.36 2010/08/12 03:22:41 sarrazip Exp $
     main-window.cpp - Input and conjugation window
 
     verbiste - French conjugation system
@@ -31,6 +31,8 @@
 #include <time.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <math.h>
+#include <errno.h>
 
 #include <libintl.h>
 
@@ -40,6 +42,7 @@
 #include <iostream>
 #include <fstream>
 #include <sstream>
+#include <algorithm>
 
 #define _(x) gettext(x)
 
@@ -63,6 +66,10 @@ static GtkWidget *showPronounsCB = NULL;
 static GtkWidget *useFrenchDictCB = NULL;
 static GtkWidget *useItalianDictCB = NULL;
 static GtkWidget *resultNotebook = NULL;
+static GtkWidget *diceScale = NULL;
+
+static const double DICE_SCALE_MIN = 0.80;
+static const double DICE_SCALE_MAX = 1.00;
 
 static const gint SP = 2;  // default spacing for the GTK+ boxes
 static bool trace = getenv("TRACE") != NULL;
@@ -83,9 +90,9 @@ void sync_config();
 class Catena
 {
 public:
-    Catena(gchar *p) : ptr(p) {}
+    Catena(gchar *p) : ptr(p) {}  // p allowed to be NULL
     ~Catena() { g_free(ptr); }
-    gchar *get() const { return ptr; }
+    gchar *get() const { return ptr; }  // returned pointer may be NULL
     gint atoi() const { return ptr == NULL ? INT_MIN : static_cast<gint>(::atoi(ptr)); }
     bool startsWithDigit() const { return ptr != NULL && ::isdigit(ptr[0]); }
 private:
@@ -152,7 +159,7 @@ quit()
     ss << resultWinHeight;
     set_config_string("Preferences/ResultWindowHeight", ss.str().c_str());
 
-    // Save the state of the checkboxes.
+    // Save the state of the checkboxes, etc.
     bool active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showPronounsCB));
     set_config_string("Preferences/ShowPronouns", active ? "1" : "0");
     active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useFrenchDictCB));
@@ -160,6 +167,16 @@ quit()
     active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useItalianDictCB));
     set_config_string("Preferences/UseItalianDict", active ? "1" : "0");
 
+    assert(diceScale != NULL);
+    gdouble minDiceCoefficient = gtk_range_get_value(GTK_RANGE(diceScale));
+    if (minDiceCoefficient < DICE_SCALE_MIN)
+        minDiceCoefficient = DICE_SCALE_MIN;
+    else if (minDiceCoefficient > DICE_SCALE_MAX)
+        minDiceCoefficient = DICE_SCALE_MAX;
+    ss.str("");
+    ss << int(floor(minDiceCoefficient * 1000));  // save as int to avoid locale problems
+    set_config_string("Preferences/MinDiceCoefficient", ss.str().c_str());
+
     sync_config();  // commit changes to file
 
     gtk_main_quit();
@@ -752,6 +769,14 @@ onShowPronounsToggled(GtkToggleButton *, gpointer)
 }
 
 
+static
+void
+onDiceScaleValueChanged(GtkRange *, gpointer)
+{
+    onConjugateButton(NULL, NULL);
+}
+
+
 static
 void
 onUseFrenchDictToggled(GtkToggleButton *, gpointer)
@@ -933,6 +958,7 @@ showResultWin()
         addAccentedCharButton(optionsBox, "\xc3\xb9"); // in HTML: &ugrave;
         addAccentedCharButton(optionsBox, "\xc3\xbb"); // in HTML: &ucirc;
 
+        gtk_box_pack_start(GTK_BOX(optionsBox), gtk_vseparator_new(), FALSE, FALSE, 0);
 
         useFrenchDictCB = gtk_check_button_new_with_mnemonic(_("Search Fr_ench"));
                                         // 'F' is already shortcut for "Fermer" button in French interface
@@ -958,12 +984,47 @@ showResultWin()
         gtk_notebook_set_scrollable(GTK_NOTEBOOK(resultNotebook), TRUE);
 
 
-
         /*
             Create a button box at the bottom.
         */
         GtkWidget *bottomBox = gtk_hbox_new(FALSE, SP);
 
+        GtkWidget *diceBeforeLabel = gtk_label_new_with_mnemonic(_("_Spelling: tolerant"));
+
+        diceScale = gtk_hscale_new_with_range(DICE_SCALE_MIN, DICE_SCALE_MAX, 0.01);
+        gtk_range_set_update_policy(GTK_RANGE(diceScale), GTK_UPDATE_DISCONTINUOUS);
+        gtk_scale_set_draw_value(GTK_SCALE(diceScale), FALSE);  // don't show number
+        gtk_scale_set_digits(GTK_SCALE(diceScale), 2);
+
+        Catena coef(get_config_string("Preferences/MinDiceCoefficient"));
+        if (trace)
+            cout << "coef: " << (coef.get() ? coef.get() : "<null>") << " (" << (void *) coef.get() << ")" << endl;
+        errno = 0;
+        long d = (coef.get() != NULL ? strtol(coef.get(), NULL, 10) : 1000);
+        if (trace)
+        {
+            int e = errno;
+            cout << "d=" << d << ", errno=" << e << endl;
+            errno = e;
+        }
+        double minDiceCoefficient = d / 1000.0;  // saved as int to avoid locale problems
+        if (errno == ERANGE || minDiceCoefficient < DICE_SCALE_MIN || minDiceCoefficient > DICE_SCALE_MAX)
+            minDiceCoefficient = 1.0;
+        gtk_range_set_value(GTK_RANGE(diceScale), minDiceCoefficient);
+
+        g_signal_connect(G_OBJECT(diceScale), "value-changed",
+                                    G_CALLBACK(onDiceScaleValueChanged), NULL);
+
+        GtkWidget *diceAfterLabel = gtk_label_new_with_mnemonic(_("strict"));
+
+        gtk_label_set_mnemonic_widget(GTK_LABEL(diceBeforeLabel), diceScale);
+
+        gtk_box_pack_start(GTK_BOX(bottomBox), diceBeforeLabel, FALSE, FALSE, 0);
+        gtk_box_pack_start(GTK_BOX(bottomBox), diceScale, TRUE, TRUE, 0);
+        gtk_box_pack_start(GTK_BOX(bottomBox), diceAfterLabel, FALSE, FALSE, 0);
+
+        gtk_box_pack_start(GTK_BOX(bottomBox), gtk_vseparator_new(), FALSE, FALSE, 0);
+
         GtkWidget *closeButton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
         gtk_box_pack_end(GTK_BOX(bottomBox), closeButton, FALSE, FALSE, 0);
         g_signal_connect(G_OBJECT(closeButton), "clicked",
@@ -1054,8 +1115,33 @@ createTableCell(const VVS &latin1Tense,
 }
 
 
-static void deconjugate(FrenchVerbDictionary *fvd, const string &utf8UserText,
-                        const string &lowerCaseUserText, bool includePronouns, size_t &numPages);
+class WaitCursor
+{
+public:
+    WaitCursor()
+    {
+        if (resultWin == NULL || resultWin->window == NULL)
+            return;
+
+        GdkCursor *cursor = gdk_cursor_new(GDK_WATCH);
+        gdk_window_set_cursor(resultWin->window, cursor);
+        gdk_cursor_unref(cursor);
+    }
+
+    ~WaitCursor()
+    {
+        if (resultWin == NULL || resultWin->window == NULL)
+            return;
+
+        gdk_window_set_cursor(resultWin->window, NULL);
+    }
+};
+
+
+static size_t deconjugate(FrenchVerbDictionary &fvd, const string &utf8UserText,
+                          const string &lowerCaseLatin1UserText, bool includePronouns);
+static size_t deconjugateFuzzyMatches(FrenchVerbDictionary &fvd, const string &utf8UserText,
+                                      const string &lowerCaseLatin1UserText, bool includePronouns);
 static void finishProcessingText(const string &utf8UserText, size_t numPages);
 
 
@@ -1066,58 +1152,72 @@ processText(const string &utf8UserText)
     It must not contain a newline character('\n').
 */
 {
-    showResultWin();
+    showResultWin();  // initializes resultWin if not already done
+
+    WaitCursor wc;  // must be used after resultWin initialized; destructor removes wait cursor
 
     gtk_entry_set_text(GTK_ENTRY(verbEntry), utf8UserText.c_str());
     gtk_editable_select_region(GTK_EDITABLE(verbEntry), 0, -1);
 
     clearResultNotebook();
 
-    // Convert to Latin-1 using any dictionary.
-    string lowerCaseUserText = frenchDict->utf8ToLatin1(tolowerUTF8(utf8UserText));
+    size_t numFrenchPages = 0, numItalianPages = 0;
 
-    assert(showPronounsCB != NULL);
-    assert(useFrenchDictCB != NULL);
-    assert(useItalianDictCB != NULL);
+    if (!utf8UserText.empty())
+    {
+        // Convert to Latin-1 using any dictionary.
+        string lowerCaseLatin1UserText = frenchDict->utf8ToLatin1(tolowerUTF8(utf8UserText));
 
-    bool includePronouns = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showPronounsCB));
-    bool useFrenchDict   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useFrenchDictCB));
-    bool useItalianDict  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useItalianDictCB));
-    size_t numPages = 0;
-    if (useFrenchDict)
-        deconjugate(frenchDict, utf8UserText, lowerCaseUserText, includePronouns, numPages);
-    if (useItalianDict)
-        deconjugate(italianDict, utf8UserText, lowerCaseUserText, includePronouns, numPages);
+        assert(showPronounsCB != NULL);
+        assert(useFrenchDictCB != NULL);
+        assert(useItalianDictCB != NULL);
 
-    finishProcessingText(utf8UserText, numPages);
+        bool includePronouns = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showPronounsCB));
+        bool useFrenchDict   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useFrenchDictCB));
+        bool useItalianDict  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(useItalianDictCB));
+        if (useFrenchDict)
+        {
+            numFrenchPages += deconjugate(*frenchDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+            if (numFrenchPages == 0)
+                numFrenchPages += deconjugateFuzzyMatches(*frenchDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+        }
+        if (useItalianDict)
+        {
+            numItalianPages += deconjugate(*italianDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+            if (numItalianPages == 0)
+                numItalianPages += deconjugateFuzzyMatches(*italianDict, utf8UserText, lowerCaseLatin1UserText, includePronouns);
+        }
+    }
+
+    finishProcessingText(utf8UserText, numFrenchPages + numItalianPages);
 }
 
 
-// Asks the FrenchVerbDictionary to deconjugate lowerCaseUserText,
+// Asks the FrenchVerbDictionary to deconjugate lowerCaseLatin1UserText,
 // then creates a notebook page for each resulting conjugation.
+// Returns the number of pages created in the notebook.
 //
-void
-deconjugate(FrenchVerbDictionary *fvd, const string &utf8UserText,
-            const string &lowerCaseUserText, bool includePronouns, size_t &numPages)
+size_t
+deconjugate(FrenchVerbDictionary &fvd, const string &utf8UserText,
+            const string &lowerCaseLatin1UserText, bool includePronouns)
 {
     if (trace)
-        cout << "deconjugate(fvd=" << fvd << " (" << fvd->getLanguageCode()
+        cout << "deconjugate(fvd=@" << &fvd << " (" << fvd.getLanguageCode()
              << "), utf8UserText='" << utf8UserText
-             << "', lowerCaseUserText='" << lowerCaseUserText
+             << "', lowerCaseLatin1UserText='" << lowerCaseLatin1UserText
              << "', includePronouns=" << includePronouns
              << ")" << endl;
 
-    g_return_if_fail(fvd != NULL);
-
-    bool isItalian = (fvd->getLanguageCode() == "it");
+    bool isItalian = (fvd.getLanguageCode() == "it");
 
     /*
         For each possible deconjugation, take the infinitive form and
         obtain its complete conjugation.
     */
     vector<InflectionDesc> v;
-    fvd->deconjugate(lowerCaseUserText, v);
+    fvd.deconjugate(lowerCaseLatin1UserText, v);
 
+    size_t numPages = 0;  // counts number of pages added to notebook
     string prevLatin1Infinitive, prevTemplateName;
 
     for (vector<InflectionDesc>::const_iterator it = v.begin();
@@ -1126,7 +1226,7 @@ deconjugate(FrenchVerbDictionary *fvd, const string &utf8UserText,
         const InflectionDesc &d = *it;
 
         VVVS latin1Conjug;
-        getConjugation(*fvd, d.infinitive, d.templateName, latin1Conjug, includePronouns);
+        getConjugation(fvd, d.infinitive, d.templateName, latin1Conjug, includePronouns);
 
         if (latin1Conjug.size() == 0           // if no tenses
             || latin1Conjug[0].size() == 0     // if no infinitive tense
@@ -1146,7 +1246,7 @@ deconjugate(FrenchVerbDictionary *fvd, const string &utf8UserText,
         if (latin1Infinitive == prevLatin1Infinitive && d.templateName == prevTemplateName)
             continue;
 
-        string label = fvd->latin1ToUTF8(latin1Infinitive) + " (" + fvd->getLanguageCode() + ")";
+        string label = fvd.latin1ToUTF8(latin1Infinitive) + " (" + fvd.getLanguageCode() + ")";
         ResultPage *rp = appendResultPage(label);
         numPages++;
 
@@ -1170,7 +1270,7 @@ deconjugate(FrenchVerbDictionary *fvd, const string &utf8UserText,
                 continue;
 
             GtkWidget *cell = createTableCell(
-                                latin1Tense, utf8TenseName, utf8UserText, fvd);
+                                latin1Tense, utf8TenseName, utf8UserText, &fvd);
             gtk_table_attach(GTK_TABLE(rp->table), cell,
                                 col, col + 1, row, row + 1,
                                 GTK_FILL, GTK_FILL,
@@ -1183,6 +1283,138 @@ deconjugate(FrenchVerbDictionary *fvd, const string &utf8UserText,
         prevLatin1Infinitive = latin1Infinitive;
         prevTemplateName = d.templateName;
     }
+
+    return numPages;
+}
+
+
+// See http://en.wikipedia.org/wiki/Dice_coefficient
+//
+static double
+diceCoefficient(const string &a, const string &b)
+{
+    size_t lenA = a.length();
+    size_t lenB = b.length();
+    if (lenA == 0 && lenB == 0)  // special case: two empty strings
+        return 1.0;
+
+    string bCopy = b;
+    size_t numCommonChars = 0;
+
+    for (size_t i = 0; i < lenA; ++i)
+    {
+        char aChar = a[i];
+        for (size_t j = 0; j < lenB; ++j)
+        {
+            if (aChar == bCopy[j])
+            {
+                ++numCommonChars;
+                bCopy[j] = '\0';
+                break;
+            }
+        }
+    }
+
+    return (2 * numCommonChars) / double(lenA + lenB);
+}
+
+
+inline bool
+isConsonant(char c)
+{
+    static const char *vowelsInLatin1 = "aeiouy\xe1\xe2\xe0\xe9\xea\xe8\xeb\xed\xee\xef\xf3\xf4\xfa\xfb\xf9\xfc";
+    return strchr(vowelsInLatin1, c) == NULL;
+}
+
+
+struct Score
+{
+    double coefficient;  // 0..1 (1 = same)
+    string verbName;
+
+    Score(double c, const string &v) : coefficient(c), verbName(v) {}
+
+    static bool decreasingCoefficientOrder(const Score &a, const Score &b)
+    {
+        if (a.coefficient != b.coefficient)
+            return a.coefficient > b.coefficient;
+        return a.verbName < b.verbName;  // use dictionary order
+    }
+};
+
+
+static void
+fuzzyMatch(FrenchVerbDictionary &fvd,
+           const string &latin1String,
+           double minDiceCoefficient,
+           vector<string> &latin1Alternatives)
+{
+    if (trace)
+        cout << "fuzzyMatch('" << latin1String << "', " << minDiceCoefficient << "):\n";
+
+    if (latin1String.empty())
+        return;
+
+    char initial = latin1String[0];
+    bool isInitConsonant = isConsonant(initial);
+    vector<Score> scores;
+
+    for (VerbTable::const_iterator it = fvd.beginKnownVerbs();
+                                   it != fvd.endKnownVerbs(); ++it)
+    {
+        const string &verbName = it->first;
+        assert(!verbName.empty());  // empty verb names unexpected in dictionary
+
+        if (isInitConsonant && verbName[0] != initial)
+            continue;
+
+        double coef = diceCoefficient(verbName, latin1String);
+
+        if (coef >= minDiceCoefficient)  // if similar enough
+        {
+            if (trace)
+                cout << "  MATCH\t" << coef << "\t" << verbName << "\n";
+            scores.push_back(Score(coef, verbName));
+        }
+    }
+
+    sort(scores.begin(), scores.end(), Score::decreasingCoefficientOrder);
+
+    for (vector<Score>::const_iterator it = scores.begin();
+                                       it != scores.end(); ++it)
+    {
+        if (trace)
+            cout << "  RET\t" << it->coefficient << "\t" << it->verbName << "\n";
+        latin1Alternatives.push_back(it->verbName);
+    }
+}
+
+
+size_t
+deconjugateFuzzyMatches(FrenchVerbDictionary &fvd, const string &utf8UserText,
+                        const string &lowerCaseLatin1UserText, bool includePronouns)
+{
+    // Get minimum Dice coefficient from GUI's spelling tolerance slider.
+    //
+    assert(diceScale != NULL);
+    gdouble minDiceCoefficient = gtk_range_get_value(GTK_RANGE(diceScale));
+    assert(minDiceCoefficient >= 0.0 && minDiceCoefficient <= 1.0);
+    if (minDiceCoefficient == 1.0)  // if not using fuzzy matching
+        return 0;
+
+    // Get a list of verbs that are similar to lowerCaseLatin1UserText.
+    //
+    vector<string> latin1Alternatives;
+    fuzzyMatch(fvd, lowerCaseLatin1UserText, minDiceCoefficient, latin1Alternatives);
+
+    // Show these verbs.
+    //
+    size_t numPages = 0;
+    for (vector<string>::const_iterator it = latin1Alternatives.begin();
+                                        it != latin1Alternatives.end(); ++it)
+        numPages += deconjugate(fvd, utf8UserText, *it, includePronouns);
+
+    return numPages;
 }
 
 
