@@ -1,4 +1,4 @@
-/*  $Id: conjugation.h,v 1.10 2010/07/02 04:04:49 sarrazip Exp $
+/*  $Id: conjugation.h,v 1.11 2010/07/11 02:56:15 sarrazip Exp $
     conjugation.h - Generic conjugation interface
 
     verbiste - French conjugation system
@@ -46,7 +46,7 @@ typedef std::vector<VVS> VVVS;
 */
 void getConjugation(const verbiste::FrenchVerbDictionary &fvd,
                         const std::string &infinitive,
-                        const string &tname,
+                        const std::string &tname,
                         VVVS &dest,
                         bool includePronouns = false);
 
