@@ -1,4 +1,4 @@
-/*  $Id: FrenchVerbDictionary.cpp,v 1.38 2010/07/04 05:49:51 sarrazip Exp $
+/*  $Id: FrenchVerbDictionary.cpp,v 1.41 2010/08/12 03:50:05 sarrazip Exp $
     FrenchVerbDictionary.cpp - Dictionary of verbs and conjugation templates
 
     verbiste - French conjugation system
@@ -223,6 +223,24 @@ FrenchVerbDictionary::FrenchVerbDictionary(
 }
 
 
+FrenchVerbDictionary::FrenchVerbDictionary(bool includeWithoutAccents)
+                                                throw (std::logic_error)
+  : conjugSys(),
+    knownVerbs(),
+    aspirateHVerbs(),
+    inflectionTable(),
+    toUTF8(),
+    toLatin1(),
+    verbTrie(*this),
+    langCode("fr")
+{
+    string conjFN, verbsFN;
+    getXMLFilenames(conjFN, verbsFN, langCode);
+
+    init(conjFN, verbsFN, includeWithoutAccents);
+}
+
+
 void
 FrenchVerbDictionary::init(const string &conjugationFilename,
                             const string &verbsFilename,
@@ -247,6 +265,10 @@ FrenchVerbDictionary::init(const string &conjugationFilename,
 
     loadConjugationDatabase(conjugationFilename.c_str(), includeWithoutAccents);
     loadVerbDatabase(verbsFilename.c_str(), includeWithoutAccents);
+
+    if (trace)
+        cout << "FrenchVerbDictionary::init: trie takes "
+             << verbTrie.computeMemoryConsumption() << " bytes\n";
 }
 
 
@@ -505,7 +527,7 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
 
         string verbRadical(infinitive, 0, lenInfinitive - lenTermination);
 
-        insertVerbRadicalInTrie(verbRadical, tname);
+        insertVerbRadicalInTrie(verbRadical, tname, verbRadical);
 
         if (includeWithoutAccents)
         {
@@ -515,7 +537,7 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
             for (vector<string>::const_iterator it = unaccentedVariants.begin();
                                                 it != unaccentedVariants.end(); ++it)
             {
-                insertVerbRadicalInTrie(*it, tname);
+                insertVerbRadicalInTrie(*it, tname, verbRadical);  // pass correct verb radical as 3rd argument
             }
         }
     }
@@ -527,9 +549,11 @@ FrenchVerbDictionary::readVerbs(xmlDocPtr doc,
 
 void
 FrenchVerbDictionary::insertVerbRadicalInTrie(
-                        const string &verbRadical, const string &tname)
+                                    const std::string &verbRadical,
+                                    const std::string &tname,
+                                    const std::string &correctVerbRadical)
 {
-    vector<string> **templateListPtr =
+    vector<TrieValue> **templateListPtr =
                             verbTrie.getUserDataPointer(verbRadical);
     assert(templateListPtr != NULL);
 
@@ -538,11 +562,11 @@ FrenchVerbDictionary::insertVerbRadicalInTrie(
     // empty vector of template names.
     //
     if (*templateListPtr == NULL)
-        *templateListPtr = new vector<string>();
+        *templateListPtr = new vector<TrieValue>();
 
     // Associate the given template name to the given verb radical.
     //
-    (*templateListPtr)->push_back(tname);
+    (*templateListPtr)->push_back(TrieValue(tname, correctVerbRadical));
 }
 
 
@@ -712,24 +736,28 @@ void
 FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
                         const string &conjugatedVerb,
                         string::size_type index,
-                        const vector<std::string> *templateList) const throw()
+                        const vector<TrieValue> *templateList) const throw()
 {
+    if (trace)
+        cout << "VerbTrie::onFoundPrefixWithUserData: start: conjugatedVerb='" << conjugatedVerb << "'" << endl;
+
     assert(templateList != NULL);
     if (results == NULL)
         return;
 
-    string radical(conjugatedVerb, 0, index);
-    string term(conjugatedVerb, index);
+    const string radical(conjugatedVerb, 0, index);
+    const string term(conjugatedVerb, index);
 
     /*
         'templateList' contains the names of conjugated templates that might
         apply to the conjugated verb.  We check each of them to see if there
         is one that accepts the given termination 'term'.
     */
-    for (vector<string>::const_iterator i = templateList->begin();
-                                                i != templateList->end(); i++)
+    for (vector<TrieValue>::const_iterator i = templateList->begin();
+                                           i != templateList->end(); i++)
     {
-        const string &tname = *i;
+        const TrieValue &trieValue = *i;
+        const string &tname = trieValue.templateName;
         const TemplateInflectionTable &ti =
                                 fvd.inflectionTable.find(tname)->second;
         TemplateInflectionTable::const_iterator j = ti.find(term);
@@ -750,9 +778,25 @@ FrenchVerbDictionary::VerbTrie::onFoundPrefixWithUserData(
         {
             const ModeTensePersonNumber &mtpn = *k;
 
-            string infinitive = radical + templateTerm;
-                // the infinitive of the conjugated verb is formed from its
-                // radical part and from the termination of the template name
+            string infinitive = trieValue.correctVerbRadical + templateTerm;
+                // The infinitive of the conjugated verb is formed from its
+                // (correct) radical part and from the termination of the template name.
+                // Correct means with the proper accents. This allows the user
+                // to type "etaler" without the acute accent on the first "e"
+                // and obtain the conjugation for the correct verb, which has
+                // that accent.
+
+            if (trace)
+                cout << "VerbTrie::onFoundPrefixWithUserData: radical='"
+                    << radical << "', templateTerm='" << templateTerm
+                    << "', tname='" << tname
+                    << "', correctVerbRadical='" << trieValue.correctVerbRadical
+                    << "', mtpn=("
+                    << mtpn.mode << ", "
+                    << mtpn.tense << ", "
+                    << (unsigned) mtpn.person << ", "
+                    << mtpn.plural << ", "
+                    << mtpn.correct << ")\n";
 
             results->push_back(InflectionDesc(infinitive, tname, mtpn));
                 // the InflectionDesc object is an analysis of the
